/*
 *   Copyright (C) 2007 Aaron Seigo <aseigo@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "ciavc.h"

#include <QDateTime>
#include <QTimer>

#include <KDebug>
#include <KLocale>
#include <KUrl>

#include <syndication/item.h>

CiaVcEngine::CiaVcEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent, args)
{
    setMinimumPollingInterval(10000);
    setPollingInterval(60000);
}

CiaVcEngine::~CiaVcEngine()
{
}

void CiaVcEngine::init()
{
}

bool CiaVcEngine::updateSourceEvent(const QString &project)
{
    Syndication::Loader * loader = Syndication::Loader::create();
    connect(loader, SIGNAL(loadingComplete(Syndication::Loader*, Syndication::FeedPtr, Syndication::ErrorCode)),
            this, SLOT(processProject(Syndication::Loader*, Syndication::FeedPtr, Syndication::ErrorCode)));
    loader->loadFrom(QString("http://cia.vc/stats/project/%1/.rss?ver=2&limit=5").arg(project));

    return false;
}

bool CiaVcEngine::sourceRequestEvent(const QString &name)
{
    updateSourceEvent(name);

    return true;
}

void CiaVcEngine::processProject(Syndication::Loader* loader, Syndication::FeedPtr feed, Syndication::ErrorCode error)
{
    Q_UNUSED(loader)
    if (error != Syndication::Success) {
        kDebug() << "Error: " << error << ", the syndication did not work out";
        QString errorText;
        switch (error) {
            case Syndication::Aborted :
                errorText = i18n("Data download was aborted");
                break;
            case Syndication::Timeout :
                errorText = i18n("Data download timed out" );
                break;
            case Syndication::UnknownHost :
                errorText = i18n("The hostname could not be resolved");
                break;
            case Syndication::FileNotFound :
                errorText = i18n("404: File not found");
                break;
            case Syndication::OtherRetrieverError :
                errorText = i18n("Retriever Error");
                break;
            case Syndication::InvalidXml :
                errorText = i18n("Invalid XML");
                break;
            case Syndication::XmlNotAccepted :
                errorText = i18n("XML not accepted by parser");
                break;
            case Syndication::InvalidFormat :
                errorText = i18n("XML parsing failed");
                break;
            default :
                errorText = i18n("Invalid Syndication::ErrorCode");
        }

        setData(I18N_NOOP("Error"), I18N_NOOP("Syndication Error"), errorText);
        return;
    }

    //TODO: implement domains in DataEngine
    QString domain = feed->title();
    kDebug() << "received " << feed->items().count() << " item(s) for " << domain;
    foreach (const Syndication::ItemPtr& item, feed->items()) {
        //FIXME: this is ugly and should be fixed in how DataSources work
        //       is this a candidate for domains? or for a DataSource QVariant
        //       so DataSources can be the data() of another DataSource? hm.
        setData(feed->title(), item->title(), item->description());
    }
}

#include "ciavc.moc"
